/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * This header defines preprocessor utilities.  (You may have seen
 * macros similar to these in an old C++ "generic" header, and of
 * course the Boost preprocessor header is much more capable.)
 *
 * These macros should be treated as equivalent to their definitions,
 * rather than encapsulating something.  You may replace them at need.
 */

#ifndef VRPSX_PREPROCESSOR_H
#define VRPSX_PREPROCESSOR_H 1


/* Concatenates the given preprocessor tokens
 * after performing preprocessor evaluation on them.
 */
#define VRPSX_CAT2(A, B)  VRPSX_CAT2_UNEVALUATED(A, B)
#define VRPSX_CAT3(A, B, C)  VRPSX_CAT3_UNEVALUATED(A, B, C)
#define VRPSX_CAT4(A, B, C, D)  VRPSX_CAT4_UNEVALUATED(A, B, C, D)
#define VRPSX_CAT5(A, B, C, D, E)  VRPSX_CAT5_UNEVALUATED(A, B, C, D, E)
#define VRPSX_CAT6(A, B, C, D, E, F)  VRPSX_CAT6_UNEVALUATED(A, B, C, D, E, F)
#define VRPSX_CAT7(A, B, C, D, E, F, G)  \
    VRPSX_CAT7_UNEVALUATED(A, B, C, D, E, F, G)

/* Concatenates the given preprocessor tokens
 * WITHOUT performing preprocessor evaluation on them.
 */
#define VRPSX_CAT2_UNEVALUATED(A, B)  A ## B
#define VRPSX_CAT3_UNEVALUATED(A, B, C)  A ## B ## C
#define VRPSX_CAT4_UNEVALUATED(A, B, C, D)  A ## B ## C ## D
#define VRPSX_CAT5_UNEVALUATED(A, B, C, D, E)  A ## B ## C ## D ## E
#define VRPSX_CAT6_UNEVALUATED(A, B, C, D, E, F)  A ## B ## C ## D ## E ## F
#define VRPSX_CAT7_UNEVALUATED(A, B, C, D, E, F, G)  \
    A ## B ## C ## D ## E ## F ## G


/* Performs preprocessor evaluation and then converts to a string literal.
 */
#define VRPSX_STRINGIZE(S)  VRPSX_STRINGIZE_UNEVALUATED(S)

/* Converts to a string literal WITHOUT preprocessor evaluation.
 */
#define VRPSX_STRINGIZE_UNEVALUATED(S)  # S


#endif /* VRPSX_PREPROCESSOR_H */
